using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Rendering;
using UnityEngine.Experimental.Rendering;

[ExecuteInEditMode]
public class MyScriptableRenderPipeline : RenderPipelineAsset
{
    protected override IRenderPipeline InternalCreatePipeline()
    {
        return new MyScriptableRenderPipelineInstance();
    }
}

/// ̃vWFNgɍ쐬 ScriptableRenderPipeline( SRP )ł
public class MyScriptableRenderPipelineInstance : RenderPipeline
{
    private CullResults cull;
    private ScriptableCullingParameters cullingParams;
    private CommandBuffer cmd;
    private ShaderPassName zPrepass = new ShaderPassName("ZPrepass");
    private ShaderPassName basicPass = new ShaderPassName("BasicPass");

    ///
    public List <CommandBuffer> zPrepassCommandBuffers = new List<CommandBuffer>();
    public List<CommandBuffer> actualCommandBuffers = new List<CommandBuffer>();
    public List<CommandBuffer> shadowCommandBuffers = new List<CommandBuffer>();

    private static MyScriptableRenderPipelineInstance instance;

    public static MyScriptableRenderPipelineInstance Instance
    {
        get { return instance; }
    }

    /// <summary>
    /// RXgN^
    /// </summary>
    public MyScriptableRenderPipelineInstance()
    {
        instance = this;
    }

    /// <summary>
    /// ۂ̕`揈
    /// </summary>
    public override void Render(ScriptableRenderContext context, Camera[] cameras)
    {
        base.Render(context, cameras);
        if (cmd == null)
        {
            cmd = new CommandBuffer();
        }
        int idx = 0;
        foreach (var camera in cameras)
        {
            // Culling܂
            if (!CullResults.GetCullingParameters(camera, out cullingParams))
                continue;
            CullResults.Cull(ref cullingParams, context,ref cull);

            // JɊւShaderp[^[ZbgAbv܂
            context.SetupCameraProperties(camera);

            // ʂNA܂
            cmd.Clear();
            cmd.ClearRenderTarget(true, true, Color.black,1.0f);
            context.ExecuteCommandBuffer(cmd);

            // Directional Light̒lݒ肵܂
            SetUpDirectionalLightParam(cull.visibleLights);

            // CommandBufferɓĂ΁c
            if (zPrepassCommandBuffers != null && idx == 0) {
                ExecuteCommandBufferList(context,zPrepassCommandBuffers);
            }
            // LN^[@ZPrepassŕ`悵܂
            DrawCharacter(context, camera, zPrepass, SortFlags.CommonOpaque);
            // BGBasicPassŕ`悵܂
            DrawBg(context, camera);

            // CommandBufferɓĂ΁c
            if (actualCommandBuffers != null && idx == 0) {
                ExecuteCommandBufferList(context, actualCommandBuffers);
            }
            // LN^[BasicPassŕ`悵܂
            DrawCharacter(context, camera, basicPass, SortFlags.OptimizeStateChanges);

            // Ă...
            if (shadowCommandBuffers != null && idx == 0)
            {
                ExecuteCommandBufferList(context, shadowCommandBuffers);
            }
            // Ōɉe`悵܂
            DrawShadow(context, camera);

            // `eR~bg܂
            context.Submit();
            ++idx;
        }
    }

    // R}hobt@[̃Xgs
    private void ExecuteCommandBufferList(ScriptableRenderContext context, List<CommandBuffer> cmdBuffers)
    {
        if (cmdBuffers == null) { return; }
        foreach (var cmdBuffer in cmdBuffers)
        {
            context.ExecuteCommandBuffer(cmdBuffer);
        }
    }


    // w肳ꂽ@PassŃLN^[`悵܂
    private void DrawCharacter(ScriptableRenderContext context, Camera camera, ShaderPassName pass,SortFlags sortFlags)
    {
        var settings = new DrawRendererSettings(camera, pass);
        settings.sorting.flags = sortFlags;

        var filterSettings = new FilterRenderersSettings(true)
        {
            renderQueueRange = RenderQueueRange.transparent,
            layerMask = 1 << LayerDefine.CHARA
        };
        context.DrawRenderers(cull.visibleRenderers, ref settings, filterSettings);
    }

    // w肳ꂽeBG`悵܂
    private void DrawBg(ScriptableRenderContext context, Camera camera)
    {
        var settings = new DrawRendererSettings(camera, basicPass);
        settings.sorting.flags = SortFlags.CommonOpaque;

        var filterSettings = new FilterRenderersSettings(true)
        {
            renderQueueRange = RenderQueueRange.opaque,
            layerMask = 1 << LayerDefine.BG
        };
        context.DrawRenderers(cull.visibleRenderers, ref settings, filterSettings);
    }
    // w肳ꂽeŉe`悵܂
    private void DrawShadow(ScriptableRenderContext context, Camera camera)
    {
        var settings = new DrawRendererSettings(camera, basicPass);
        settings.sorting.flags = SortFlags.CommonTransparent;

        var filterSettings = new FilterRenderersSettings(true)
        {
            renderQueueRange = RenderQueueRange.transparent,
            layerMask = 1 << LayerDefine.SHADOW
        };
        context.DrawRenderers(cull.visibleRenderers, ref settings, filterSettings);
    }


    // Directional Light̓eShaderɔf܂
    private void SetUpDirectionalLightParam(List<VisibleLight> visibleLights)
    {
        if( visibleLights.Count <= 0 ){
            return;
        }
        foreach( var visibleLight in visibleLights)
        {
            if (visibleLight.lightType == LightType.Directional)
            {
                Vector4 dir = -visibleLight.localToWorld.GetColumn(2) ;
                Shader.SetGlobalVector(ShaderNameHash.LightColor0, visibleLight.finalColor);
                Shader.SetGlobalVector(ShaderNameHash.WorldSpaceLightPos0, new Vector4(dir.x,dir.y,dir.z,0.0f) );
                break;
            }
        }
    }

    // j̏
    public override void Dispose()
    {
        base.Dispose();
        instance = null;
        if (cmd != null)
        {
            cmd.Dispose();
            cmd = null;
        }
    }
}
